//
//  CompositionalViewController.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/12/28.
//

import UIKit

class CompositionalViewController: UIViewController {
    private enum Section {
        case all
        case iOS
        case iPadOS
        case watchOS
        case tvOS
        case macOS
    }

    @IBOutlet private weak var collectionView: UICollectionView! {
        didSet {
            collectionView.collectionViewLayout = compositionalLayout
            var snapshot = NSDiffableDataSourceSnapshot<Section, Device>()
            snapshot.appendSections([.all])
            snapshot.appendItems(Device.all, toSection: .all)
            dataSource.apply(snapshot)
        }
    }

    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, Device> = {
        let cellRegistration: UICollectionView.CellRegistration<DeviceCell, Device> =
            .init { cell, indexPath, device in
                cell.device = device
            }
        return .init(collectionView: collectionView,
                     cellProvider: { collectionView, indexPath, device in
                        collectionView.dequeueConfiguredReusableCell(using: cellRegistration,
                                                                     for: indexPath,
                                                                     item: device)})
    }()

    private let compositionalLayout = UICollectionViewCompositionalLayout { sectionIndex, layoutEnvironment in
        let leadingItem = NSCollectionLayoutItem(
            layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(0.7),
                heightDimension: .fractionalHeight(1.0)))
        let trailingItem = NSCollectionLayoutItem(
            layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(1.0),
                heightDimension: .fractionalHeight(0.3)))
        let trailingGroup = NSCollectionLayoutGroup.vertical(
            layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(0.3),
                heightDimension: .fractionalHeight(1.0)),
            subitem: trailingItem,
            count: 2)
        let nestedGroup = NSCollectionLayoutGroup.horizontal(
            layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(1.0),
                heightDimension: .fractionalHeight(0.4)),
            subitems: [leadingItem, trailingGroup])
        return NSCollectionLayoutSection(group: nestedGroup)
    }

}
