//
//  Device.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/12/04.
//

import UIKit

struct Device: Hashable {
    enum OS {
        case iOS
        case iPadOS
        case watchOS
        case tvOS
        case macOS

        var color: UIColor {
            switch self {
            case .iOS:
                return .systemBlue
            case .iPadOS:
                return .systemRed
            case .watchOS:
                return .systemPurple
            case .tvOS:
                return .systemGreen
            case .macOS:
                return .systemGray
            }
        }
    }

    let model: String
    let os: OS

    var image: UIImage {
        UIImage(systemName: model)!
    }

    var color: UIColor {
        os.color
    }

    static var all: [Device] {
        iOSDevices + iPadOSDevices + watchOSDevices + tvOSDevices + macOSDevices
    }

    static var iOSDevices: [Device] {
        [
            .init(model: "iphone", os: .iOS),
            .init(model: "iphone.homebutton", os: .iOS),
            .init(model: "ipod", os: .iOS),
            .init(model: "ipodshuffle.gen1", os: .iOS),
            .init(model: "ipodshuffle.gen2", os: .iOS),
            .init(model: "ipodshuffle.gen3", os: .iOS),
            .init(model: "ipodshuffle.gen4", os: .iOS),
            .init(model: "ipodtouch", os: .iOS),
        ]
    }

    static var iPadOSDevices: [Device] {
        [
            .init(model: "ipad", os: .iPadOS),
            .init(model: "ipad.homebutton", os: .iPadOS),
        ]
    }

    static var watchOSDevices: [Device] {
        [
            .init(model: "applewatch", os: .watchOS),
        ]
    }

    static var tvOSDevices: [Device] {
        [
            .init(model: "appletv", os: .tvOS),
        ]
    }

    static var macOSDevices: [Device] {
        [
            .init(model: "macmini", os: .macOS),
            .init(model: "macpro.gen1", os: .macOS),
            .init(model: "macpro.gen2", os: .macOS),
            .init(model: "macpro.gen3", os: .macOS),
            .init(model: "macpro.gen3.server", os: .macOS),
        ]
    }
}
