//
//  DeviceCell.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/12/04.
//

import UIKit

class DeviceCell: UICollectionViewCell {

    private let imageView: UIImageView

    override init(frame: CGRect) {
        let imageView = UIImageView()
        imageView.translatesAutoresizingMaskIntoConstraints = false
        imageView.contentMode = .scaleAspectFit
        self.imageView = imageView
        super.init(frame: frame)
        contentView.addSubview(imageView)
        NSLayoutConstraint.activate([
            imageView.leadingAnchor.constraint(equalTo: contentView.leadingAnchor),
            imageView.trailingAnchor.constraint(equalTo: contentView.trailingAnchor),
            imageView.topAnchor.constraint(equalTo: contentView.topAnchor),
            imageView.bottomAnchor.constraint(equalTo: contentView.bottomAnchor),
        ])
        self.layer.borderColor = UIColor.separator.cgColor
        self.layer.borderWidth = 1.0
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    var device: Device? {
        didSet {
            imageView.image = device?.image
            imageView.tintColor = device?.color
        }
    }

    override func updateConfiguration(using state: UICellConfigurationState) {
        var configuration = UIBackgroundConfiguration.clear()
        if state.isHighlighted || state.isSelected {
            configuration.backgroundColor = .gray
            if state.isHighlighted {
                configuration.backgroundColorTransformer = .init { $0.withAlphaComponent(0.3) }
            }
        }
        self.backgroundConfiguration = configuration
    }
}
