//
//  DiffableSectionCollectionViewController.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/12/28.
//

import UIKit

class DiffableCollectionViewController: UIViewController {
    private enum Section {
        case all
        case iOS
        case iPadOS
        case watchOS
        case tvOS
        case macOS
    }

    @IBOutlet private weak var collectionView: UICollectionView! {
        didSet {
            collectionView.collectionViewLayout = UICollectionViewFlowLayout()
            dataSource.apply(sectionSnapshot)
        }
    }

    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, Device> = {
        let cellRegistration: UICollectionView.CellRegistration<DeviceCell, Device> =
            .init { cell, indexPath, device in
                cell.device = device
            }
        return .init(collectionView: self.collectionView,
                     cellProvider: { collectionView, indexPath, device in
                        collectionView.dequeueConfiguredReusableCell(using: cellRegistration,
                                                                     for: indexPath,
                                                                     item: device)
                     }
        )
    }()

    private var isAll: Bool = false

    private var sectionSnapshot: NSDiffableDataSourceSnapshot<Section, Device> {
        var snapshot = NSDiffableDataSourceSnapshot<Section, Device>()
        snapshot.appendSections(
            [.iOS, .iPadOS, .watchOS, .tvOS, .macOS]
        )
        snapshot.appendItems(Device.iOSDevices,
                             toSection: .iOS)
        snapshot.appendItems(Device.iPadOSDevices,
                             toSection: .iPadOS)
        snapshot.appendItems(Device.watchOSDevices,
                             toSection: .watchOS)
        snapshot.appendItems(Device.tvOSDevices,
                             toSection: .tvOS)
        snapshot.appendItems(Device.macOSDevices,
                             toSection: .macOS)
        return snapshot
    }

    @IBAction
    func toggleSort(_ sender: UIBarButtonItem) {
        isAll.toggle()
        var snapshot: NSDiffableDataSourceSnapshot<Section, Device>
        if isAll {
            snapshot = .init()
            snapshot.appendSections([.all])
            snapshot.appendItems(Device.all,
                                 toSection: .all)
        } else {
            snapshot = sectionSnapshot
        }
        dataSource.apply(snapshot,
                         animatingDifferences: true)
    }
}
