//
//  ListCollectionViewController.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/12/05.
//

import UIKit

class ListCollectionViewController: UIViewController {
    private enum Section {
        case all
    }

    @IBOutlet private weak var collectionView: UICollectionView! {
        didSet {
            let configuration = UICollectionLayoutListConfiguration(appearance: .plain)
            collectionView.collectionViewLayout = UICollectionViewCompositionalLayout.list(using: configuration)
            var snapshot = NSDiffableDataSourceSnapshot<Section, Device>()
            snapshot.appendSections([.all])
            snapshot.appendItems(Device.all, toSection: .all)
            dataSource.apply(snapshot)
        }
    }

    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, Device> = {
        let cellRegistration: UICollectionView.CellRegistration<UICollectionViewListCell, Device> =
            .init { cell, indexPath, device in
                var configuration = cell.defaultContentConfiguration()
                configuration.image = device.image
                configuration.text = device.model
                cell.contentConfiguration = configuration
                cell.tintColor = device.color
                cell.accessories = [.disclosureIndicator()]
            }
        return .init(collectionView: self.collectionView,
                     cellProvider: { collectionView, indexPath, device in
                        return collectionView.dequeueConfiguredReusableCell(using: cellRegistration,
                                                                            for: indexPath,
                                                                            item: device)
                     }
        )
    }()
}
