//
//  ExampleWidget.swift
//  ExampleWidget
//
//  Created by Yutaro Muta on 2020/12/06.
//

import WidgetKit
import SwiftUI

@main
struct ExampleWidgetBundle: WidgetBundle {
    @WidgetBundleBuilder
    var body: some Widget {
        StaticWidget()
    }
}

struct Entry: TimelineEntry {
    let date: Date
    let relevance: TimelineEntryRelevance?
    let devices: [Device]
}

struct WidgetEntryView: View {
    @Environment(\.widgetFamily)
    var family: WidgetFamily
    var devices: [Device]

    var body: some View {
        switch family {
        case .systemSmall:
            SingleDeviceView(device: devices[0])
                .widgetURL(devices[0].url)
        case .systemMedium:
            DoudleDeviceView(devices: devices)
        case .systemLarge:
            ListDeviceView(devices: devices)
        default:
            SingleDeviceView(device: devices[0])
                .widgetURL(devices[0].url)
        }
    }
}

struct DoudleDeviceView: View {
    var devices: [Device]

    var body: some View {
        HStack {
            Link(destination: devices[0].url) {
                SingleDeviceView(device: devices[0])
            }
            Divider()
            Link(destination: devices[1].url) {
                SingleDeviceView(device: devices[1])
            }
        }
    }
}

struct ListDeviceView: View {
    var devices: [Device]

    var body: some View {
        VStack(alignment: .leading) {
            Link(destination: devices[0].url) {
                HStack {
                    SingleDeviceView(device: devices[0])
                    Text(devices[0].model)
                }
            }
            Divider()
            Link(destination: devices[1].url) {
                HStack {
                    SingleDeviceView(device: devices[1])
                    Text(devices[1].model)
                }
            }
            Divider()
            Link(destination: devices[2].url) {
                HStack {
                    SingleDeviceView(device: devices[2])
                    Text(devices[2].model)
                }
            }
        }
    }
}

extension Device.OS {
    var score: Float {
        switch self {
        case .iOS:
            return Float(5.0)
        case .iPadOS:
            return Float(4.0)
        case .watchOS:
            return Float(3.0)
        case .tvOS:
            return Float(2.0)
        case .macOS:
            return Float(1.0)
        }
    }
}
