//
//  Device.swift
//  WidgetExample
//
//  Created by Yutaro Muta on 2020/12/04.
//

import SwiftUI

struct Device: Hashable, Identifiable {
    enum OS {
        case iOS
        case iPadOS
        case watchOS
        case tvOS
        case macOS

        var color: Color {
            switch self {
            case .iOS:
                return .blue
            case .iPadOS:
                return .red
            case .watchOS:
                return .purple
            case .tvOS:
                return .green
            case .macOS:
                return .gray
            }
        }
    }

    let model: String
    let os: OS

    var id: String {
        model
    }

    var image: Image {
        Image(systemName: model)
    }

    var color: Color {
        os.color
    }

    var url: URL {
        URL(string: "example://\(model)")!
    }

    init?(from url: URL) {
        switch url.absoluteString {
        case "example://iphone":
            self = .init(model: "iphone", os: .iOS)
        case "example://iphone.homebutton":
            self = .init(model: "iphone.homebutton", os: .iOS)
        case "example://ipod":
            self = .init(model: "ipod", os: .iOS)
        case "example://ipodshuffle.gen1":
            self = .init(model: "ipodshuffle.gen1", os: .iOS)
        case "example://ipodshuffle.gen2":
            self = .init(model: "ipodshuffle.gen2", os: .iOS)
        case "example://ipodshuffle.gen3":
            self = .init(model: "ipodshuffle.gen3", os: .iOS)
        case "example://ipodshuffle.gen4":
            self = .init(model: "ipodshuffle.gen4", os: .iOS)
        case "example://ipodtouch":
            self = .init(model: "ipodtouch", os: .iOS)
        case "example://ipad":
            self = .init(model: "ipad", os: .iPadOS)
        case "example://ipad.homebutton":
            self = .init(model: "ipad.homebutton", os: .iPadOS)
        case "example://applewatch":
            self = .init(model: "applewatch", os: .watchOS)
        case "example://appletv":
            self = .init(model: "appletv", os: .tvOS)
        case "example://macmini":
            self = .init(model: "macmini", os: .macOS)
        case "example://macpro.gen1":
            self = .init(model: "macpro.gen1", os: .macOS)
        case "example://macpro.gen2":
            self = .init(model: "macpro.gen2", os: .macOS)
        case "example://macpro.gen13":
            self = .init(model: "macpro.gen3", os: .macOS)
        case "example://macpro.gen3.server":
            self = .init(model: "macpro.gen3.server", os: .macOS)
        default:
            return nil
        }
    }

    init(model: String, os: Device.OS) {
        self.model = model
        self.os = os
    }

    static var all: [Device] {
        iOSDevices + iPadOSDevices + watchOSDevices + tvOSDevices + macOSDevices
    }

    static var iOSDevices: [Device] {
        [
            .init(model: "iphone", os: .iOS),
            .init(model: "iphone.homebutton", os: .iOS),
            .init(model: "ipod", os: .iOS),
            .init(model: "ipodshuffle.gen1", os: .iOS),
            .init(model: "ipodshuffle.gen2", os: .iOS),
            .init(model: "ipodshuffle.gen3", os: .iOS),
            .init(model: "ipodshuffle.gen4", os: .iOS),
            .init(model: "ipodtouch", os: .iOS),
        ]
    }

    static var iPadOSDevices: [Device] {
        [
            .init(model: "ipad", os: .iPadOS),
            .init(model: "ipad.homebutton", os: .iPadOS),
        ]
    }

    static var watchOSDevices: [Device] {
        [
            .init(model: "applewatch", os: .watchOS),
        ]
    }

    static var tvOSDevices: [Device] {
        [
            .init(model: "appletv", os: .tvOS),
        ]
    }

    static var macOSDevices: [Device] {
        [
            .init(model: "macmini", os: .macOS),
            .init(model: "macpro.gen1", os: .macOS),
            .init(model: "macpro.gen2", os: .macOS),
            .init(model: "macpro.gen3", os: .macOS),
            .init(model: "macpro.gen3.server", os: .macOS),
        ]
    }
}
