//
//  ViewController.swift
//  MultipleWindows
//
//  Created by Kouki Saito on 2020/12/20.
//

import UIKit

struct Item: Hashable {
    let name: String
}

class CollectionViewController: UICollectionViewController, UICollectionViewDragDelegate {

    private enum Section {
        case main
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        collectionView.dragDelegate = self

        var snapshot = NSDiffableDataSourceSnapshot<Section, Item>()
        snapshot.appendSections([.main])
        snapshot.appendItems([
                                Item(name: "iphone"),
                                Item(name: "ipod"),
                                Item(name: "ipad")
        ], toSection: .main)
        dataSource.apply(snapshot)

        collectionView.dataSource = dataSource

        collectionView.collectionViewLayout = UICollectionViewCompositionalLayout { sectionIndex, layoutEnvironment in
            let item = NSCollectionLayoutItem(layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(0.5),
                heightDimension: .fractionalHeight(1.0)))
            let group = NSCollectionLayoutGroup.horizontal(layoutSize: NSCollectionLayoutSize(
                widthDimension: .fractionalWidth(1.0),
                heightDimension: .fractionalWidth(0.5)
            ), subitems: [item])

            let section = NSCollectionLayoutSection(group: group)
            return section
        }
    }

    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, Item> =
        .init(collectionView: self.collectionView,
              cellProvider: { collectionView, indexPath, item in
                let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "Cell", for: indexPath) as! ImageCollectionViewCell
                cell.image.image = UIImage(systemName: item.name)
                return cell
              }
        )

    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let viewController = segue.destination as? DetailViewController,
           let selectedIndexPath = collectionView.indexPathsForSelectedItems?.first,
           let selectedItem = dataSource.itemIdentifier(for: selectedIndexPath) {
            viewController.item = selectedItem
        }
    }

    func collectionView(_ collectionView: UICollectionView, itemsForBeginning session: UIDragSession, at indexPath: IndexPath) -> [UIDragItem] {
        guard let item = dataSource.itemIdentifier(for: indexPath) else { return [] }

        let userActivity = NSUserActivity(activityType: "com.example.app.activity")
        userActivity.userInfo = ["name": item.name]

        let itemProvider = NSItemProvider(object: UIImage(systemName: item.name)!)
        itemProvider.registerObject(userActivity, visibility: .all)
        let dragItem = UIDragItem(itemProvider: itemProvider)

        return [dragItem]
    }
}


class ImageCollectionViewCell: UICollectionViewCell {
    @IBOutlet weak var image: UIImageView!
}

class DetailViewController: UIViewController {
    @IBOutlet weak var image: UIImageView!
    var item: Item!

    override func viewDidLoad() {
        image.image = UIImage(systemName: item.name)
    }
}
