//
//  ViewController.swift
//  Sidebars
//
//  Created by Kouki Saito on 2020/12/20.
//

import UIKit

class SidebarUIKit: UICollectionViewController {
    private enum Section {
        case main
    }
    
    private struct Item: Hashable {
        let title: String
    }

    override func viewDidLoad() {
        super.viewDidLoad()
                
        var snapshot = NSDiffableDataSourceSnapshot<Section, Item>()
        snapshot.appendSections([.main])
        snapshot.appendItems([
                                Item(title: "First"),
                                Item(title: "Second"),
                                Item(title: "Third")
        ], toSection: .main)
        dataSource.apply(snapshot)

        collectionView.dataSource = dataSource
        
        let configuration = UICollectionLayoutListConfiguration(appearance: .sidebar)
        collectionView.collectionViewLayout = UICollectionViewCompositionalLayout.list(using: configuration)
    }
    
    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, Item> =
        .init(collectionView: self.collectionView,
              cellProvider: { collectionView, indexPath, item in

                let rowRegistration = UICollectionView.CellRegistration<UICollectionViewListCell, Item> {
                    (cell, indexPath, item) in
                    var contentConfiguration = UIListContentConfiguration.sidebarCell()
                    contentConfiguration.text = item.title
                    contentConfiguration.image = UIImage(systemName: "ipad")
                    
                    cell.contentConfiguration = contentConfiguration
                }
                return collectionView.dequeueConfiguredReusableCell(using: rowRegistration, for: indexPath, item: item)
              }
        )
    
    override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        guard let item = dataSource.itemIdentifier(for: indexPath) else { return }
        let viewController = storyboard!.instantiateViewController(identifier: "Detail")
        viewController.title = item.title
        showDetailViewController(viewController, sender: nil)
    }
}


class DetailUIKit: UIViewController {
    @IBOutlet var titleLabel: UILabel!
    
    override func viewDidLoad() {
        titleLabel.text = title
    }
}

extension DetailUIKit {
    override var keyCommands: [UIKeyCommand]? {
        return [
            UIKeyCommand(title: "コンテンツを保存", action: #selector(save), input: "s", modifierFlags: .command),
            UIKeyCommand(title: "新しく開く", action: #selector(new), input: "n", modifierFlags: .command),
            UIKeyCommand(title: "検索", action: #selector(save), input: "f", modifierFlags: .command),
        ]
    }
    
    @objc func save() { print("Save") }
    @objc func new() { print("New") }
    @objc func search() { print("Search") }
}
